#ifndef WuEnclosure_h
#define WuEnclosure_h

#include "capd/capdlib.h"
#include "LocalVField.h"

///////////////////////////
// class: WuEnclosure
//
// Purpose: The class provides an enclosure of the unstable manifold of 
// a fixed point of a map f. The estimate is performed based on the vector field of the ODE.
//
// The manifold is parametreised by 'x'. 
// The aim is to obtain an enclosure expressed as a function
//   image(x)
//
// The bound on the unstable manifold is expressed by cones
//    L*||x|| - ||y|| > 0
//
// The ODE is considered in local coordinates where 'x' is the expanding coordinate and 'y' is the contracting coordinate.
//
// Members:
//   *f         - Pointer to the vector field in local coordinates
//   domain     - The parametrisation image(x) has its domain; i.e. x \in domain
//   L          - slope of the cone
//   fixedPoint - fixed point enclosure in local coordinates 
class WuEnclosure
{
public:
	LocalVField *f;
	interval L;
	IVector domain,fixedPoint;
	
	C0Rect2Set image(const IVector &x) const;
	IVector localImage(const IVector &x) const;
	C0Rect2Set operator ()(const IVector &x) const {return image(x);}
	
	WuEnclosure(LocalVField &f, IVector domain, interval L);
	WuEnclosure(LocalVField &f, IVector Bu, interval L,IVector Bs);
	
	void EstablishManifold(IVector Bu,IVector Bs);
};

#endif
