#include "isolatingBlock.h"
#include "projections.h"

using namespace std;

interval largestDiagonalCoeff(IMatrix A)
{
	int k=A.numberOfColumns();
	interval a=A[0][0];
	for(int i=1;i<k;i++)
	{
		if(A[i][i].left()>a) a=A[i][i];
	}
	return a;
}

interval smallestDiagonalCoeff(IMatrix A)
{
	int k=A.numberOfColumns();
	interval a=A[0][0];
	for(int i=1;i<k;i++)
	{
		if(A[i][i].right()<a) a=A[i][i];
	}
	return a;
}

void neglectRotation(IMatrix &R)
{
	if(R.numberOfColumns()==2)
	{
		interval a=R[0][1].mid();
		R[0][1]=R[0][1]-a;
		R[1][0]=R[1][0]+a;
	}
}

bool satisfiesIsolatingBlockCondition(LocalVField &F, IMatrix DF, int x_dimension,interval r1, interval r2)
{
	int k=DF.numberOfColumns();
	capd::vectalg::EuclNorm<IVector,IMatrix> Norm;
	
	IMatrix dF_xx=Fxx(DF,x_dimension);
	IMatrix dF_xy=Fxy(DF,x_dimension);
	IMatrix dF_yx=Fyx(DF,x_dimension);
	IMatrix dF_yy=Fyy(DF,x_dimension);
	
	IMatrix C11(x_dimension,x_dimension);
	IMatrix C22(k-x_dimension,k-x_dimension);
	
	for(int i=0;i<x_dimension;i++) C11[i][i]=dF_xx[i][i].mid();
	for(int i=0;i<k-x_dimension;i++) C22[i][i]=dF_yy[i][i].mid();
	
	IMatrix R11=dF_xx-C11;
	IMatrix R22=dF_yy-C22;
	IMatrix R12=dF_xy;
	IMatrix R21=dF_yx;
	
	neglectRotation(R11);
	neglectRotation(R22);
	
	IVector zero(k);
	if(! (smallestDiagonalCoeff(C11)-Norm(R11)*r1-Norm(R12)*r2-Norm(F(zero)) > 0.0) ) return 0;
	if(! ( largestDiagonalCoeff(C22)+Norm(R21)*r1+Norm(R22)*r2+Norm(F(zero)) < 0.0) ) return 0;
	
	return 1;
}