#include "ratecond.h"
#include "projections.h"
#include "utils.h"

///////////////
// This function computes the bound on the logarithmic min-norm of a matrix
interval ml(const IMatrix &A)
{
	capd::vectalg::EuclLNorm<IVector,IMatrix> l;
	return -l(-A);
}

bool satisfiesRateConditions(const IMatrix &DF,const interval &L,const int &x_dimension)
{	
	capd::vectalg::EuclLNorm<IVector,IMatrix> l;
	capd::vectalg::EuclNorm <IVector,IMatrix> euclNorm;
	
	IMatrix dF_xx=Fxx(DF,x_dimension);
	IMatrix dF_xy=Fxy(DF,x_dimension);
	IMatrix dF_yx=Fyx(DF,x_dimension);
	IMatrix dF_yy=Fyy(DF,x_dimension);
		
	interval mu1=l(dF_yy)+euclNorm(dF_yx)/L;
	interval mu2=l(dF_yy)+euclNorm(dF_xy)/L;
	
	interval xi =ml(dF_xx)-euclNorm(dF_xy)/L;
	interval xiP=xi;
	
	if(! (mu1<0.0)) return 0;
	if(! (0.0<xiP)) return 0;
	if(! (mu2<xi) ) return 0;
	return 1;
}

bool satisfiesExtendedRateConditions(const IMatrix &DF,const interval &M,const int &x_theta_dimension)
{
	capd::vectalg::EuclLNorm<IVector,IMatrix> l;
	capd::vectalg::EuclNorm <IVector,IMatrix> euclNorm;
	
	IMatrix dF_xTheta_xTheta=Fxx(DF,x_theta_dimension);
	IMatrix dF_xTheta_y=Fxy(DF,x_theta_dimension);
	IMatrix dF_y_xTheta=Fyx(DF,x_theta_dimension);
	IMatrix dF_y_y=Fyy(DF,x_theta_dimension);
		
	interval mu=l(dF_y_y)+M*euclNorm(dF_y_xTheta);
	interval xi =ml(dF_xTheta_xTheta)-euclNorm(dF_xTheta_y)/M;
	
	if(! (mu<xi)) return 0;
	return 1;
}
