#ifndef ratecond_h
#define ratecond_h

#include "capd/capdlib.h"
#include <iostream>
using namespace std;
using namespace capd;

///////////////////
// function: satisfiesRateConditions()
//
// This function verifies if rate conditins needed to establish the enclosure of Wu hold.
// The conditions are verified based on the bound on the derivative DF on the whole 
// investigated set.
//
// variables:
//  DF - bound on the derivative
//  L  - The Lipschitz constant for the cones
//  k  - order of the rate conditions
//
// Note:
// This function is written for the particular setting of a problem where coordinates are:
//   (x,y)
//   x - unstable coordinate
//   y - stable coordinate (or center-stable)
bool satisfiesRateConditions(const IMatrix &DF,const interval &L,const int &x_dimension);

////////////////////
// Function: satisfiesExtendedRateConditions()
// 
// The purpose of this function is to verify the assumptions of the theorem
// from the paper, which concerns the bounds on the manifold with respect to the parameter.
// If we verify the below considerd rate conditions on a set, which encloses 
// the manifold. Our manifold is assumed to be in the form of a graph:
//    (theta,x,w(theta,x))
// and if below rate conditions are satisfied (meaning that the function returns
// "true"), then 
//    ||dw/d theta|| <= 1/L
// To verify the assumptions, the vector fiels needs to be on the space extended by parameter.
// Our coordinates then are:
//    (theta,x,y)
// where
//   theta - is the parameter
//       x - is the expanding coordinate
//       y - is the contracting coordinate
// The matrix DF is the derivative of a vector field in such extended coordinates, on the set
// which contains the investigated manifold.
bool satisfiesExtendedRateConditions(const IMatrix &DF,const interval &L,const int &x_theta_dimension);

#endif